# encoding: utf-8

from urllib.parse import parse_qsl

from django.apps import apps
from django.contrib.admin.utils import prepare_lookup_value
from django.http import Http404, JsonResponse
from django.views.generic.list import BaseListView

from cool.core.utils import get_search_results
from cool.settings import cool_settings


class CoolAutocompleteJsonView(BaseListView):
    """
    外键自动提示view
    """
    paginate_by = 20

    def get(self, request, *args, **kwargs):
        """
        Return a JsonResponse with search results of the form:
        {
            results: [{id: "123", text: "foo"}],
            pagination: {more: true}
        }
        """
        app_label = request.GET['app_label']
        model_name = request.GET['model_name']
        limit_choices_to = request.GET.get('limit_choices_to', None)
        to_field_name = request.GET.get('to_field_name', 'pk')
        term = request.GET.get('term', '')

        if not self.has_perm(request, app_label, model_name):
            return JsonResponse({'error': '403 Forbidden'}, status=403)
        try:
            self.model = apps.get_model(app_label, model_name)
        except LookupError:
            return JsonResponse({'error': '403 Forbidden'}, status=403)

        queryset = self.get_queryset()

        filters = dict()
        if limit_choices_to:
            for key, value in parse_qsl(limit_choices_to):
                filters[key] = prepare_lookup_value(key, value)
        if filters:
            queryset = queryset.filter(**filters)

        get_search_fields = getattr(self.model, 'get_search_fields')
        search_fields = None
        if get_search_fields and callable(get_search_fields):
            search_fields = get_search_fields()
        if not search_fields:
            raise Http404('%s must have get_search_fields for the autocomplete_view.' % type(self.model).__name__)
        queryset, search_use_distinct = get_search_results(queryset, term, search_fields, self.model)
        if search_use_distinct:
            queryset = queryset.distinct()
        context = self.get_context_data(object_list=queryset)
        return JsonResponse({
            'results': [
                {'id': str(getattr(obj, to_field_name)), 'text': str(obj)}
                for obj in context['object_list']
            ],
            'pagination': {'more': context['page_obj'].has_next()},
        })

    @classmethod
    def has_perm(cls, request, app_label, model_name):
        if cool_settings.ADMIN_AUTOCOMPLETE_CHECK_PERM:
            return (
                    request.user.has_perm('%s.%s_%s' % (app_label, 'view', model_name)) or
                    request.user.has_perm('%s.%s_%s' % (app_label, 'change', model_name))
            )
        else:
            return request.user.is_staff and request.user.is_active



# encoding: utf-8

from cool.views.error_code import ErrorCode
from cool.views.exceptions import CoolAPIException, CoolPermissionAPIException
from cool.views.mixins import (
    AddMixin, DeleteMixin, EditMixin, ExtManyToOneMixin, PageMixin,
    SearchListMixin,
)
from cool.views.response import ResponseData
from cool.views.serializer import BaseSerializer
from cool.views.sites import ViewSite
from cool.views.utils import get_api_doc, get_api_doc_html, get_api_info
from cool.views.view import CoolBFFAPIView

__all__ = [
    'ErrorCode', 'CoolAPIException', 'CoolPermissionAPIException', 'ResponseData',
    'get_api_doc', 'get_api_doc_html', 'get_api_info',
    'BaseSerializer', 'ViewSite', 'CoolBFFAPIView',
    'SearchListMixin', 'PageMixin',  'AddMixin', 'DeleteMixin', 'EditMixin', 'ExtManyToOneMixin',
]



# encoding: utf-8

from django.contrib.auth.models import Group, Permission
from django.db import DatabaseError, models
from django.db.models.manager import EmptyManager
from django.utils.functional import cached_property

from cool.model.cache import model_cache


class ModelChangeMixin:
    """
    监控字段修改
    """

    def __setattr__(self, name, value):
        if not name.startswith('_') and name in self.__dict__:
            self._to_change(name, value)
        object.__setattr__(self, name, value)

    def _to_change(self, name, value):
        if name not in self.changed_map:
            self.changed_map[name] = getattr(self, name, None)
        if value == self.changed_map[name]:
            del self.changed_map[name]

    @cached_property
    def changed_map(self):
        return {}

    def get_origin(self, name):
        return self.changed_map.get(name, getattr(self, name))


class ModelFieldChangeMixin(ModelChangeMixin):
    """
    监控 Model field 变更
    """
    def __setattr__(self, name, value):
        if not name.startswith('_') and name in self.__dict__:
            opts = self._meta
            if not hasattr(opts, '_attname_map'):
                _attname_map = {}
                _name_map = {}
                _auto_now_names = set()
                fields = opts._get_fields(reverse=False, include_hidden=True)
                for field in fields:
                    # model fields defined directly and not primary key
                    if isinstance(field, models.Field) and not field.primary_key:
                        _attname_map[field.attname] = field
                        _name_map[field.name] = field
                        if isinstance(field, models.DateField) and field.auto_now:
                            _auto_now_names.add(field.name)

                opts._attname_map = _attname_map
                opts._name_map = _name_map
                opts._auto_now_names = _auto_now_names
            if name in opts._attname_map:
                field = opts._attname_map.get(name)
                self._to_change(name, value)
                self.changed_fields.add(field.name)
            elif name in opts._name_map:
                field = opts._name_map.get(name)
                if isinstance(field, models.ForeignKey):
                    if name in self.__dict__ or getattr(self, field.attname) is None:
                        self._to_change(name, value)
                else:
                    self._to_change(name, value)

        object.__setattr__(self, name, value)

    @cached_property
    def changed_fields(self):
        return set()

    def save_changed(self, using=None):
        """
        值保存修改被修改字段
        """
        if self.changed_fields:
            self.changed_fields.update(getattr(self._meta, '_auto_now_names', set()))
            self.save(force_update=True, update_fields=list(self.changed_fields), using=using)
            self.changed_fields.clear()
            self.changed_map.clear()


class ModelCacheMixin:
    """
    将 Model 以主键和唯一键缓存到 cache 中
    """

    _MODEL_WITH_CACHE = True
    _MODEL_CACHE_TTL = 600

    @classmethod
    def get_queryset(cls):
        return cls.objects

    @classmethod
    def get_obj_by_pk_from_cache(cls, pk):
        """
        通过主键获取对象（优先走缓存）
        """
        if cls._MODEL_WITH_CACHE:
            return model_cache.get(cls, pk, ttl=cls._MODEL_CACHE_TTL)
        else:
            return cls.get_queryset().filter(pk=pk).first()

    @classmethod
    def get_objs_by_pks_from_cache(cls, pks):
        """
        通过主键批量获取对象（优先走缓存）
        """
        if cls._MODEL_WITH_CACHE:
            return model_cache.get_many(cls, pks, ttl=cls._MODEL_CACHE_TTL)
        else:
            return {obj.pk: obj for obj in cls.get_queryset().filter(pk__in=pks)}

    @classmethod
    def flush_cache_by_pk(cls, pk):
        """
        清空主键缓存
        """
        if cls._MODEL_WITH_CACHE and pk is not None:
            model_cache.delete(cls, pk)

    @classmethod
    def flush_cache_by_pks(cls, pks):
        """
        批量清空主键缓存
        """
        if cls._MODEL_WITH_CACHE and pks is not None:
            model_cache.delete(cls, pks)

    @classmethod
    def get_obj_by_unique_key_from_cache(cls, **kwargs):
        """
        通过有唯一索引的字段获取对象（优先走缓存）
        """
        assert len(kwargs) == 1
        key, value = list(kwargs.items())[0]
        if cls._MODEL_WITH_CACHE:
            return model_cache.get(cls, value, key, ttl=cls._MODEL_CACHE_TTL)
        else:
            return cls.get_queryset().filter(**{key: value}).first()

    @classmethod
    def get_objs_by_unique_keys_from_cache(cls, **kwargs):
        """
        通过有唯一索引的字段批量获取对象（优先走缓存）
        """
        assert len(kwargs) == 1
        key, value = list(kwargs.items())[0]
        if cls._MODEL_WITH_CACHE:
            return model_cache.get_many(cls, value, key, ttl=cls._MODEL_CACHE_TTL)
        else:
            return {obj.pk: obj for obj in cls.get_queryset().filter(**{"%s__in" % key: value})}

    @classmethod
    def flush_cache_by_unique_key(cls, **kwargs):
        """
        清空唯一索引缓存
        """
        assert len(kwargs) == 1
        key, value = list(kwargs.items())[0]
        if cls._MODEL_WITH_CACHE and value is not None:
            model_cache.delete(cls, value, key)

    @classmethod
    def flush_cache_by_unique_keys(cls, **kwargs):
        """
        批量清空唯一索引缓存
        """
        assert len(kwargs) == 1
        key, value = list(kwargs.items())[0]
        if cls._MODEL_WITH_CACHE and value is not None:
            model_cache.delete_many(cls, value, key)

    def flush_cache(self):
        """
        清空对象所有缓存缓存
        """
        self.flush_cache_by_pk(self.pk)
        for field in self._meta.fields:
            if field.unique:
                self.flush_cache_by_unique_key(**field.get_filter_kwargs_for_object(self))


class SearchModelMixin:
    """
    为 Model 自动生成 search_fields 可用于 Admin 搜索及外键智能提示搜索
    """
    @classmethod
    def _gen_search_fields(cls):
        if not hasattr(cls, '_search_fields'):
            ret = set()
            for field in cls._meta.fields:
                if not field.db_index and not field.unique:
                    continue
                if isinstance(field, models.ForeignKey) or isinstance(field, models.ManyToManyField):
                    continue
                if isinstance(field, models.CharField):
                    ret.add('^%s' % field.name)
                elif isinstance(field, models.IntegerField) and not field.choices:
                    ret.add('=%s' % field.name)
            setattr(cls, '_search_fields', ret)
        return set(getattr(cls, '_search_fields'))

    @classmethod
    def get_search_fields(cls):
        """
        返回本model可以被搜索的字段集合（基类回自动将带索引的字段生成搜索字段集合）
        """
        return cls._gen_search_fields()


class BaseModel(ModelFieldChangeMixin, ModelCacheMixin, SearchModelMixin, models.Model):
    """
    Model基类，支持字段变更监控记录，主键唯一键缓存，搜索字段自动生成
    """
    class Meta:
        abstract = True
        ordering = ['-pk', ]

    @classmethod
    def get_child_field(cls, attr):
        field_names = attr.split('__')
        head_model = cls
        name = ''
        field = None
        for field_name in field_names:
            if head_model is None:
                raise RuntimeError('%s not found %s' % (cls, attr))
            field = head_model._meta.get_field(field_name)
            head_model = field.remote_field.model if field.remote_field is not None else None
            if name == '':
                name = field.verbose_name
            else:
                name += ' ' + field.verbose_name
        return isinstance(field, models.BooleanField), name

    def __str__(self):
        return '%s%s(%s)' % (self.__class__.__name__, self._meta.verbose_name, self.pk)

    def delete(self, using=None, keep_parents=False):
        ret = super().delete(using=using, keep_parents=keep_parents)
        self.flush_cache()
        return ret

    def save(self, force_insert=False, force_update=False, using=None, update_fields=None):
        if update_fields:
            fd = {f.name for f in self._meta.fields}
            update_fields = list(fd & set(update_fields))
        try:
            super().save(
                force_insert=force_insert, force_update=force_update, using=using, update_fields=update_fields
            )
        except DatabaseError as exp:
            if not str(exp).endswith('did not affect any rows.'):
                raise exp
        self.flush_cache()


class AbstractUserMixin:
    """
    自定义User表继承后可快速支持authenticate
    """
    is_staff = False
    is_active = True
    is_superuser = False
    _groups = EmptyManager(Group)
    _user_permissions = EmptyManager(Permission)

    @property
    def groups(self):
        return self._groups

    @property
    def user_permissions(self):
        return self._user_permissions

    def get_user_permissions(self, obj=None):
        return set()

    def get_group_permissions(self, obj=None):
        return set()

    def get_all_permissions(self, obj=None):
        return set()

    def has_perm(self, perm, obj=None):
        return False

    def has_perms(self, perm_list, obj=None):
        return False

    def has_module_perms(self, module):
        return False

    @property
    def is_anonymous(self):
        return False

    @property
    def is_authenticated(self):
        return True

    def get_username(self):
        try:
            return super().get_username()
        except Exception:
            return ''



# encoding: utf-8

from django.db.models.fields.related_descriptors import (
    ForwardManyToOneDescriptor, ForwardOneToOneDescriptor,
)

from cool.model import models


class ForwardCacheDescriptorMixin:
    """
    外键数据使用缓存
    """

    def get_cache_object(self, instance):
        remote_model = self.field.remote_field.model
        if not issubclass(remote_model, models.ModelCacheMixin) or not remote_model._MODEL_WITH_CACHE:
            return None
        if len(self.field.foreign_related_fields) != 1:
            return None

        val = self.field.get_local_related_value(instance)
        if len(val) != 1:
            return None
        base_filter = {
            rh_field.attname: getattr(instance, lh_field.attname)
            for lh_field, rh_field in self.field.related_fields
        }
        return remote_model.get_obj_by_unique_key_from_cache(**base_filter)

    def get_object(self, instance):
        try:
            ret = self.get_cache_object(instance)
        except Exception:
            ret = None
        if ret is not None:
            return ret
        return super().get_object(instance)


class ForwardManyToOneCacheDescriptor(ForwardCacheDescriptorMixin, ForwardManyToOneDescriptor):
    """
    多对一外键使用缓存
    """


class ForwardOneToOneCacheDescriptor(ForwardCacheDescriptorMixin, ForwardOneToOneDescriptor):
    """
    一对一外键使用缓存
    """



# encoding: utf-8

from django.apps import apps
from django.core.checks import Error, Tags, register
from django.core.exceptions import FieldError
from django.db.models import F

from cool.core.utils import construct_search


def register_checks():
    register(Tags.models)(check_models)


def check_models(app_configs, **kwargs):
    if app_configs is None:
        app_configs = apps.get_app_configs()

    errors = []
    for app_config in app_configs:
        for model in app_config.get_models():
            errors.extend(check_model(model))
    return errors


def check_model(model):
    errors = []
    errors.extend(_check_get_search_fields(model))
    return errors


def _check_get_search_fields(model):
    if not hasattr(model, 'get_search_fields'):
        return []

    # Ensure that autocomplete_search_fields returns a valid list of filters
    # for a QuerySet on that model
    failures = []
    for lookup in model.get_search_fields():
        try:
            # This only constructs the QuerySet and doesn't actually query the
            # DB, so it's fine for check phase.
            model._default_manager.filter(**{construct_search(model._default_manager, lookup): F('pk')})
        except FieldError:
            failures.append(lookup)

    if not failures:
        return []
    else:
        return [
            Error(
                "Model {app}.{model} returned bad entries for get_search_fields: {failures}".format(
                    app=model._meta.app_label,
                    model=model._meta.model_name,
                    failures=",".join(failures)
                ),
                hint="A QuerySet for {model} could not be constructed. Fix "
                     "the get_search_fields on it to return valid lookups.".format(model=model._meta.model_name),
                id='cool.E001'
            )
        ]



# encoding: utf-8
import copy
import datetime
import json
import logging
from io import BytesIO

from channels.generic.websocket import JsonWebsocketConsumer
from channels.http import AsgiRequest
from django.urls import Resolver404, get_resolver
from django.utils.functional import cached_property

from cool.settings import cool_settings


class CoolWsRequest(AsgiRequest):

    _dont_enforce_csrf_checks = True

    def __getattr__(self, item):
        if item in self.scope:
            return self.scope[item]
        else:
            return self.__getattribute__(item)


class CoolBFFAPIConsumer(JsonWebsocketConsumer):
    """
    api接口支持websocket调用
    """

    logger = logging.getLogger('cool.views')

    @cached_property
    def raw_uri(self):
        scope = copy.copy(self.scope)
        scope["method"] = "GET"
        return CoolWsRequest(scope, BytesIO(b'')).get_raw_uri()

    def receive(self, *args, **kwargs):
        self.logger.info("websocket receive %s %s %s", self.raw_uri, args, kwargs)
        super().receive(*args, **kwargs)

    def send(self, *args, **kwargs):
        super().receive(*args, **kwargs)
        self.logger.info("websocket send %s %s %s", self.raw_uri, args, kwargs)

    @classmethod
    def check_resolver_match(cls, callback, callback_args, callback_kwargs):
        from cool.views import CoolBFFAPIView
        return hasattr(callback, 'view_class') and issubclass(callback.view_class, CoolBFFAPIView)

    @classmethod
    def get_response(cls, request):
        resolver = get_resolver()
        resolver_match = resolver.resolve(request.path)
        callback, callback_args, callback_kwargs = resolver_match
        if not cls.check_resolver_match(callback, callback_args, callback_kwargs):
            raise Resolver404({'path': request.path})
        request.resolver_match = resolver_match
        response = callback(request, *callback_args, **callback_kwargs)
        return response

    @classmethod
    def create_request(cls, scope, path, data):
        try:
            data = json.dumps(data).encode('utf-8')
        except Exception:
            data = b''
        scope = copy.copy(scope)
        scope["method"] = "POST"
        scope["path"] = path
        if data:
            if 'headers' in scope:
                scope['headers'] = copy.deepcopy(scope['headers'])
            else:
                scope['headers'] = []
            scope['headers'].append((b'content-type', b"application/json"))
            scope['headers'].append((b'content-length', b"%d" % len(data)))
        return CoolWsRequest(scope, BytesIO(data))

    def receive_json(self, content, **kwargs):
        req_id = content.get(cool_settings.API_WS_REQ_ID_NAME, None)
        req_path = content.get(cool_settings.API_WS_REQ_PATH_NAME, None)
        req_data = content.get(cool_settings.API_WS_REQ_DATA_NAME, None)
        res = dict()
        request = None
        try:
            request = self.create_request(self.scope, req_path, req_data)
            response = self.get_response(request)
            res[cool_settings.API_WS_RES_DATA_NAME] = response.data
            res[cool_settings.API_WS_RES_STATUS_CODE_NAME] = response.status_code
        except Resolver404:
            res[cool_settings.API_WS_RES_STATUS_CODE_NAME] = cool_settings.API_WS_RES_STATUS_CODE_NOT_FOUND
        except Exception as exc:
            self.logger.error("websocket exception %s", self.raw_uri, exc_info=exc, extra={'request': request})
            res[cool_settings.API_WS_RES_STATUS_CODE_NAME] = cool_settings.API_WS_RES_STATUS_CODE_SERVER_ERROR

        res[cool_settings.API_WS_RES_SERVER_TIME_NAME] = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        res[cool_settings.API_WS_REQ_ID_NAME] = req_id
        self.send_json(res)

    @classmethod
    def encode_json(cls, content):
        from rest_framework.utils import encoders
        return json.dumps(content, cls=encoders.JSONEncoder)



# encoding: utf-8

from django.apps import apps
from django.contrib.auth.management import create_permissions
from django.core.management.base import BaseCommand


class Command(BaseCommand):
    """
    自动创建app增删改查权限，供Admin使用（大多用于sqlmigrate中不会对新增model创建权限）
    """
    args = '<app app ...>'
    help = 'create permissions for specified apps, or all apps if none specified'

    def handle(self, *args, **options):
        if not args:
            appconfigs = apps.get_app_configs()
        else:
            appconfigs = set()
            for arg in args:
                appconfigs.add(apps.get_app_config(arg))
        for app in appconfigs:
            create_permissions(app, options.get('verbosity', 2))



# encoding: utf-8

from django.apps import AppConfig
from django.contrib.admin.filters import FieldListFilter, ListFilter
from django.db import models
from django.utils.translation import gettext_lazy as _

from cool.admin import filters
from cool.checks import register_checks
from cool.settings import cool_settings


class CoolConfig(AppConfig):
    name = 'cool'
    verbose_name = _("Django Cool")

    def ready(self):
        register_checks()
        if cool_settings.ADMIN_FILTER_USE_SELECT:
            ListFilter.template = 'cool/admin/select_filter.html'
        if cool_settings.ADMIN_RELATED_FIELD_FILTER_USE_AUTOCOMPLETE:
            FieldListFilter.register(lambda f: f.remote_field, filters.AutocompleteFieldFilter, True)
        if cool_settings.ADMIN_DATE_FIELD_FILTER_USE_RANGE:
            FieldListFilter.register(
                lambda f: isinstance(f, models.DateTimeField), filters.DateTimeRangeFieldFilter, True
            )
            FieldListFilter.register(lambda f: isinstance(f, models.DateField), filters.DateRangeFieldFilter, True)



# encoding: utf-8
import copy
import inspect
import json
from collections import OrderedDict

from django.utils.translation import gettext_lazy as _
from rest_framework.exceptions import ValidationError
from rest_framework.fields import CharField, Field, JSONField
from rest_framework.serializers import BaseSerializer

from . import utils
from .view import ParamSerializer


class SplitCharField(CharField):
    """
    分割字符串字段，字段会用指定分隔符分割为列表
    """
    child = CharField(allow_blank=True)

    def __init__(self, **kwargs):
        self.sep = kwargs.pop('sep', ',')
        self.child = kwargs.pop('child', copy.deepcopy(self.child))

        assert not inspect.isclass(self.child), '`child` has not been instantiated.'
        assert self.child.source is None, (
            "The `source` argument is not meaningful when applied to a `child=` field. "
            "Remove `source=` from the field declaration."
        )

        super().__init__(**kwargs)
        self.child.bind(field_name='', parent=self)

    def to_internal_value(self, data):
        if data is None:
            return data
        ret = super().to_internal_value(data)
        return self.run_child_validation(ret.split(self.sep))

    def run_child_validation(self, data):
        result = []
        errors = OrderedDict()

        for idx, item in enumerate(data):
            try:
                result.append(self.child.run_validation(item))
            except ValidationError as e:
                errors[idx] = e.detail

        if not errors:
            return result
        raise ValidationError([errors])

    def to_representation(self, value):
        return self.sep.join(map(str, value))


class JSONCheckFieldBase(JSONField):
    default_error_messages = {
        'not_a_list': _('Expected a list of items but got type "{input_type}".'),
        'not_a_dict': _('Expected a dictionary of items but got type "{input_type}".'),
        'empty': _('This list may not be empty.'),
    }

    def __init__(self, *args, **kwargs):
        self.is_list = kwargs.pop('is_list', False)
        if 'help_text' not in kwargs:
            kwargs['help_text'] = json.dumps(self.format_json(), ensure_ascii=False)
        super().__init__(*args, **kwargs)
        self.param_form = self.gen_param_form(self)

    def to_internal_value(self, data):
        data = super().to_internal_value(data)
        return self.run_children_validation(data)

    @classmethod
    def merge_data(cls, bounded_form, data):
        data.update(bounded_form.cleaned_data)
        return data

    def clean_dict_data(self, data):
        bounded_form = self.param_form(data=data)
        bounded_form.is_valid()
        errors = bounded_form.errors
        if errors:
            exc = ValidationError(errors)
            raise exc
        return self.merge_data(bounded_form, data)

    def format_json(self):
        raise NotImplementedError

    @classmethod
    def gen_param_form(cls, obj):
        raise NotImplementedError

    def run_children_validation(self, data):
        if data is None:
            return data
        if self.is_list and not isinstance(data, list):
            self.fail('not_a_list', input_type=type(data).__name__)
        if not self.is_list and not isinstance(data, dict):
            self.fail('not_a_dict', input_type=type(data).__name__)
        result = []
        errors = OrderedDict()
        if self.is_list:
            for idx, item in enumerate(data):
                try:
                    result.append(self.clean_dict_data(item))
                except ValidationError as e:
                    errors[idx] = e.detail
        else:
            try:
                result = self.clean_dict_data(data)
            except ValidationError as e:
                errors = e.detail

        if not errors:
            return result
        raise ValidationError([errors])


class JSONCheckField(JSONCheckFieldBase):
    """
    json检查字段，会根据children限制对json数据进行检查
    """
    def __init__(self, *args, **kwargs):

        self.children = kwargs.pop('children', dict())
        assert isinstance(self.children, dict)
        for child in self.children.values():
            assert isinstance(child, Field)
        super().__init__(*args, **kwargs)

    def format_json(self):
        info = {key: child.label for key, child in self.children.items()}
        return [info] if self.is_list else info

    @classmethod
    def gen_param_form(cls, obj):
        form_attrs = copy.deepcopy(obj.children)

        class Meta:
            fields = list(form_attrs.keys())

        form_attrs['Meta'] = Meta
        return type(ParamSerializer)(cls.__name__ + 'ParamSerializer',  (ParamSerializer, ), form_attrs)


class SerializerField(JSONCheckFieldBase):
    """
    序列化类型字段，通过json字段解析
    """

    def __init__(self, serializer, *args, **kwargs):
        assert issubclass(serializer, BaseSerializer)
        self.serializer = serializer
        super().__init__(*args, **kwargs)

    def format_json(self):
        return utils.get_serializer_info(self.serializer(), self.is_list)

    @classmethod
    def merge_data(cls, bounded_form, data):
        data.update(bounded_form.validated_data)
        return data

    @classmethod
    def gen_param_form(cls, obj):
        return obj.serializer



# encoding: utf-8

import inspect

from django.core.cache import DEFAULT_CACHE_ALIAS, caches
from django.core.cache.backends.base import DEFAULT_TIMEOUT
from django.utils.encoding import force_str


def _is_cache_item(obj):
    return isinstance(obj, CacheItem)


class CacheItem:
    """
    缓存项目类
    """
    def __init__(self, cache=None, name=None, default_timeout=DEFAULT_TIMEOUT):
        self.cache = cache
        self.name = name
        self.default_timeout = default_timeout

    def add(self, key, value, timeout=DEFAULT_TIMEOUT):
        return self.cache.inner_call(self, 'add', key=key, value=value, timeout=timeout)

    def get(self, key, default=None):
        return self.cache.inner_call(self, 'get', key=key, default=default)

    def set(self, key, value, timeout=DEFAULT_TIMEOUT):
        return self.cache.inner_call(self, 'set', key=key, value=value, timeout=timeout)

    def touch(self, key, timeout=DEFAULT_TIMEOUT):
        return self.cache.inner_call(self, 'touch', key=key, timeout=timeout)

    def delete(self, key):
        return self.cache.inner_call(self, 'delete', key=key)

    def get_many(self, keys):
        return self.cache.inner_call(self, 'get_many', keys=keys, ret_dict_key=True)

    def get_or_set(self, key, default, timeout=DEFAULT_TIMEOUT):
        return self.cache.inner_call(self, 'get_or_set', key=key, default=default, timeout=timeout)

    def incr(self, key, delta=1):
        return self.cache.inner_call(self, 'incr', key=key, delta=delta)

    def decr(self, key, delta=1):
        return self.cache.inner_call(self, 'decr', key=key, delta=delta)

    def set_many(self, data, timeout=DEFAULT_TIMEOUT):
        return self.cache.inner_call(self, 'set_many', key_dict_fields=('data', ), data=data, timeout=timeout)

    def delete_many(self, keys):
        return self.cache.inner_call(self, 'delete_many', keys=keys)

    def call(self, func_name, **kwargs):
        return self.cache.inner_call(self, func_name, **kwargs)


class BaseCache:
    """
        缓存管理基类

        class MyCache(BaseCache):
            key_prefix = 'my_cache'
            default_timeout = 600
            item1 = CacheItem()
            item2 = CacheItem(default_timeout=60)
        cache = MyCache()
        cache.item1.set("test", 1)
        cache.item1.get("test")
    """
    key_prefix = None
    default_timeout = DEFAULT_TIMEOUT
    cache_alias = DEFAULT_CACHE_ALIAS

    def __new__(cls, *args, **kwargs):
        self = super().__new__(cls)
        api_endpoints = inspect.getmembers(self, _is_cache_item)
        for name, api in api_endpoints:
            api_cls = type(api)
            api = api_cls(self, name)
            setattr(self, name, api)
        return self

    def __init__(self):
        self.cache = caches[self.cache_alias]
        assert self.key_prefix is not None

    def get_timeout(self, item=None, timeout=DEFAULT_TIMEOUT):
        if timeout == DEFAULT_TIMEOUT:
            if _is_cache_item(item) and item.default_timeout != DEFAULT_TIMEOUT:
                timeout = item.default_timeout
            else:
                timeout = self.default_timeout
        elif timeout == 0:
            timeout = -1
        return timeout

    def item_prefix(self, item):
        return '%s:%s' % (self.key_prefix, item.name)

    def make_key(self, item, key):
        k = self.item_prefix(item)
        if isinstance(key, (tuple, list)):
            key = ':'.join(map(force_str, key))
        if key is not None:
            k = '%s:%s' % (k, key)
        return k

    def inner_call(
            self, item, func_name, *,
            key_fields=('key', ),
            keys_fields=('keys', ),
            key_dict_fields=(),
            timeout_fields=('timeout', ),
            ret_dict_key=False,
            ret_list_key=False,
            **kwargs
    ):
        for key in key_fields:
            if key in kwargs:
                kwargs[key] = self.make_key(item, kwargs[key])
        for keys in keys_fields:
            if keys in kwargs:
                kwargs[keys] = [self.make_key(item, key) for key in kwargs[keys]]
        for key_dict in key_dict_fields:
            if key_dict in kwargs and isinstance(kwargs[key_dict], dict):
                kwargs[key_dict] = {self.make_key(item, key): value for key, value in kwargs[key_dict].items()}
        for timeout in timeout_fields:
            if timeout in kwargs:
                kwargs[timeout] = self.get_timeout(item, kwargs[timeout])
        ret = getattr(self.cache, func_name)(**kwargs)

        def _get_real_key(_item_prefix, _key):
            if _item_prefix == _key:
                return None
            _item_prefix += ":"
            if not _key.startswith(_item_prefix):
                return _key
            else:
                return _key[len(_item_prefix):]

        if ret_dict_key and isinstance(ret, dict):
            item_prefix = self.item_prefix(item)
            return {_get_real_key(item_prefix, k): v for k, v in ret.items()}
        if ret_list_key and isinstance(ret, list):
            item_prefix = self.item_prefix(item)
            return [_get_real_key(item_prefix, k) for k in ret]
        return ret



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

import json
import logging
import time
from collections import OrderedDict

from django.conf import settings
from django.core.exceptions import ValidationError as CoreValidationError
from django.db.models import Model, QuerySet
from django.forms import forms
from django.http import HttpResponse
from django.template.loader import render_to_string
from django.utils.datastructures import MultiValueDict
from django.utils.encoding import force_text
from django.utils.safestring import mark_safe
from rest_framework import serializers
from rest_framework.exceptions import ValidationError as RestValidationError
from rest_framework.fields import empty
from rest_framework.serializers import ModelSerializer
from rest_framework.views import APIView

from cool.settings import cool_settings
from cool.views.error_code import ErrorCode
from cool.views.exceptions import CoolAPIException
from cool.views.options import ViewMetaclass, ViewOptions
from cool.views.param import Param
from cool.views.response import ResponseData


class ParamSerializer(serializers.Serializer):
    def __init__(self, instance=None, data=empty, files=None, is_form=True, **kwargs):
        d = MultiValueDict()
        if data is not empty:
            d.update(data)
        if files:
            d.update(files)
        if not d:
            request = kwargs.get('context', dict()).get('request', None)
            view = kwargs.get('context', dict()).get('view', None)
            if request is not None:
                d.update(request.GET)
                d.update(request.POST)
                d.update(request.FILES)
            if view is not None and hasattr(view, 'kwargs'):
                d.update(view.kwargs)
        if not is_form:
            d = d.dict()
        super().__init__(instance, d, **kwargs)
        self.is_valid()

    @property
    def cleaned_data(self):
        self.is_valid()
        data = self.validated_data.copy()
        for key, field in self.fields.items():
            if key not in data:
                data[key] = getattr(field, 'default', empty)
        return data

    def update(self, instance, validated_data):
        raise RuntimeError()

    def create(self, validated_data):
        raise RuntimeError()


class APIViewOptions(ViewOptions):
    """APIView options class

    Extend options:
    * wrappers          view's wrappers generate with the nearest-first logic,
                        this attribute will generate all parents' wrappers
    """

    def __init__(self, options=None, parent=None):
        super().__init__(options, parent)
        if not hasattr(options, 'form') and self.form is forms.Form:
            self.form = ParamSerializer
        self.wrappers = list(reversed(getattr(options, 'wrappers', ())))
        if self.parent:
            self.wrappers.extend(parent._meta.wrappers)

    def wrap_view(self, view):
        for wrapper in self.wrappers:
            view = wrapper(view)
        return view

    def gen_param_form(self, cls):
        form_attrs = dict(self.param_fields)

        class Meta:
            fields = list(form_attrs.keys())

        form_attrs['__module__'] = cls.__module__
        form_attrs['Meta'] = Meta
        cls.param_form = type(self.form)(cls.__name__ + 'ParamSerializer',  (self.form, ), form_attrs)
        cls.serializer_class = cls.param_form


class CoolBFFAPIView(APIView, metaclass=ViewMetaclass):
    """
    Backend For Frontend APIView
    """

    logger = logging.getLogger('cool.views')

    option_class = APIViewOptions
    SYSTEM_ERROR_STATUS_CODE = cool_settings.API_SYSTEM_ERROR_STATUS_CODE
    PARAM_ERROR_STATUS_CODE = cool_settings.API_PARAM_ERROR_STATUS_CODE
    SUCCESS_WITH_CODE_MSG = cool_settings.API_SUCCESS_WITH_CODE_MSG
    SHOW_PARAM_ERROR_INFO = cool_settings.API_SHOW_PARAM_ERROR_INFO
    description_template_name = 'cool/views/api_description.html'

    # 序列化类
    response_info_serializer_class = None

    # 是否返回list
    response_many = False

    @classmethod
    def get_extend_param_fields(cls):
        return ()

    # 验证请求
    def initialize_request(self, request, *args, **kwargs):
        _ = request.body
        return super().initialize_request(request, *args, **kwargs)

    def get_view_description(self, html=False):
        if not html or not self.description_template_name:
            return super().get_view_description(html)
        view_info = self.get_view_info()
        return mark_safe(render_to_string(self.description_template_name, view_info))

    @classmethod
    def get_view_info(cls):
        request_info = cls.request_info_data()
        response_info = ResponseData(
            cls.response_info_data(), success_with_code_msg=cls.SUCCESS_WITH_CODE_MSG
        ).get_response_data()
        return {
            'request_info': request_info,
            'response_info': response_info,
            'response_info_format': json.dumps(response_info, ensure_ascii=False, indent=4)
        }

    @classmethod
    def response_info_data(cls):
        """
        返回数据样例
        """
        if cls.response_info_serializer_class is not None:
            from cool.views.utils import get_serializer_info
            return get_serializer_info(cls.response_info_serializer_class(), cls.response_many)
        return None

    @classmethod
    def request_info_data(cls):
        """
        请求数据样例
        """
        from cool.views.utils import get_field_info
        ret = OrderedDict()
        serializer_class = getattr(cls, 'serializer_class', None)
        if serializer_class is not None:
            serializer = serializer_class()
            for key, field in serializer.fields.items():
                ret[key] = get_field_info(field)
        return ret

    #
    def init_params(self, request, *args, **kwargs):
        """
        兼容post和get请求
        """
        data = MultiValueDict()
        data.update(request.GET)
        data.update(request.POST)
        if hasattr(request, 'data') and isinstance(request.data, dict):
            data.update(request.data)
        data.update(kwargs)
        request.params = Param(self, request, data, request.FILES)

    def get_response(self, context):
        """
        返回数据的校验和序列化
        """
        if isinstance(context, HttpResponse):
            return context
        if isinstance(context, (Model, QuerySet)) and issubclass(self.response_info_serializer_class, ModelSerializer):
            context = self.response_info_serializer_class(context, many=self.response_many, request=self.request).data
        if not isinstance(context, ResponseData):
            context = ResponseData(context, success_with_code_msg=self.SUCCESS_WITH_CODE_MSG)
        return context.get_response()

    def check_api_permissions(self, request, *args, **kwargs):
        """
        权限校验
        """
        pass

    def view(self, request, *args, **kwargs):
        self.init_params(request, *args, **kwargs)
        self.check_api_permissions(request, *args, **kwargs)
        context = self.get_context(request, *args, **kwargs)
        return self.get_response(context)

    def get(self, request, *args, **kwargs):
        return self.view(request, *args, **kwargs)

    post = get

    def get_context(self, request, *args, **kwargs):
        """
        编写业务逻辑
        """
        raise NotImplementedError

    @classmethod
    def get_param_error_info(cls, exc):
        """
        错误代码样例
        """
        from cool.views.utils import parse_validation_error
        data = dict()
        if cls.SHOW_PARAM_ERROR_INFO:
            exc_data = parse_validation_error(exc)
            if exc_data is exc:
                data['desc'] = force_text(exc)
            else:
                data['errors'] = exc_data
        return data

    def get_exception_handler(self):
        super_exception_handler = super().get_exception_handler()

        def get_exception_handler(exc, context):
            ret = super_exception_handler(exc, context)
            if ret is None:
                ret = self.get_uncaught_exception_response(exc, context)
            return ret
        return get_exception_handler

    def handle_exception(self, exc):
        if isinstance(exc, (RestValidationError, CoreValidationError)):
            exc = CoolAPIException(
                ErrorCode.ERROR_BAD_PARAMETER,
                data=self.get_param_error_info(exc),
                status_code=self.PARAM_ERROR_STATUS_CODE
            )
        if isinstance(exc, CoolAPIException):
            return self.get_response(exc.response_data)
        return super().handle_exception(exc)

    def initial(self, request, *args, **kwargs):
        self.log_request(request, *args, **kwargs)
        return super().initial(request, *args, **kwargs)

    def finalize_response(self, request, response, *args, **kwargs):
        response = super().finalize_response(request, response, *args, **kwargs)
        self.log_response(request, response, *args, **kwargs)
        return response

    def get_uncaught_exception_response(self, exc, context):
        self.log_exception(self.request, exc, context)
        if settings.DEBUG:
            return None
        return self.get_response(ResponseData(None, ErrorCode.ERROR_SYSTEM, status_code=self.SYSTEM_ERROR_STATUS_CODE))

    def log_request(self, request, *args, **kwargs):
        import uuid
        request.start_time = time.time()
        request.uid = uuid.uuid4().hex
        self.logger.info(
            "request start %s %s %s %s %s %s",
            request.method,
            request.get_raw_uri(),
            request.uid,
            request.user,
            request.data,
            request.META
        )

    def log_response(self, request, response, *args, **kwargs):
        from rest_framework.response import Response
        from rest_framework.utils import encoders
        if isinstance(response, Response):
            try:
                data = json.dumps(response.data, ensure_ascii=False, cls=encoders.JSONEncoder)
            except Exception:
                data = str(response.data)
        elif isinstance(response, HttpResponse):
            data = response.content
        else:
            data = ''
        if response.status_code == 200 and len(data) > 1024:
            data = data[:1000]
        self.logger.info(
            "request finish %ss %s %s %s %s %s %s %s %s",
            time.time() - getattr(request, 'start_time', 0),
            request.method,
            request.get_raw_uri(),
            getattr(request, 'uid', ''),
            response.status_code,
            request.user,
            request.data,
            data,
            request.META
        )

    def log_exception(self, request, exc, context):
        self.logger.error(
            "request exception %ss %s %s %s %s %s %s",
            time.time() - getattr(request, 'start_time', 0),
            request.method,
            request.get_raw_uri(),
            getattr(request, 'uid', ''),
            request.user,
            request.data,
            request.META,
            exc_info=exc,
            extra={'request': request}
        )



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

from django import forms
from django.forms.utils import ErrorDict
from rest_framework.exceptions import ValidationError
from rest_framework.request import is_form_media_type


class TextErrorDict(ErrorDict):
    def __str__(self):
        return self.as_text()


class Param:
    """
    Wrap django-form to access parameters more conveniently
    """
    def __init__(self, view, request, request_data=None, files=None):
        """
        May raise ValidationError when checking is False when accessing parameters
        """
        self._opts = view._meta
        self._request = request
        is_form = True
        if request.method == 'POST':
            meta = request.META
            content_type = meta.get('CONTENT_TYPE', meta.get('HTTP_CONTENT_TYPE', ''))
            is_form = is_form_media_type(content_type)
        self._bounded_form = view.param_form(data=request_data, files=files, is_form=is_form)
        self._dependency = self._opts.param_dependency
        if self._opts.param_managed:
            errors = self._bounded_form.errors
            if errors:
                raise ValidationError(errors)

            self._cleaned_data = self._bounded_form.cleaned_data

            self._clean_dependency()
        else:
            self._cleaned_data = {}

    @property
    def form(self):
        return self._bounded_form

    @form.setter
    def form(self, other):
        self._bounded_form = other
        if getattr(other, 'cleaned_data', None):
            self._cleaned_data = other.cleaned_data
            self._clean_dependency()
        else:
            self._cleaned_data = {}

    def _clean_dependency(self, name=None):
        if name is None:
            names = self._cleaned_data.keys()
        else:
            names = [name]

        errors = ErrorDict()
        for name in names:
            if name in self._dependency:
                try:
                    field, pairs = self._dependency[name]
                    try:
                        _ = field.clean(self._cleaned_data[name])
                    except forms.ValidationError:
                        continue
                    for sub_name, sub_field in pairs:
                        _ = sub_field.clean(self._cleaned_data[sub_name])  # NOQA
                except forms.ValidationError as exc:
                    error_dict = TextErrorDict([(sub_name, exc.messages)])
                    errors[name] = [error_dict]
                    del self._cleaned_data[name]

        if errors:
            raise forms.ValidationError(errors)

    def __getattr__(self, name):
        if name not in self._cleaned_data:
            form = self._bounded_form
            if name not in form.fields:
                raise AttributeError(name)
            field = form.fields[name]
            value = field.widget.value_from_datadict(
                form.data, form.files, form.add_prefix(name))
            if isinstance(field, forms.FileField):
                initial = self.initial.get(name, field.initial)
                value = field.clean(value, initial)
            else:
                value = field.clean(value)
            self._cleaned_data[name] = value
            self._clean_dependency(name)

        return self._cleaned_data[name]

    def __getitem__(self, name):
        return self.__getattr__(name)

    def __setitem__(self, name, value):
        self._cleaned_data[name] = value

    def __str__(self):
        return str(self._cleaned_data)



# encoding: utf-8

from cool.model.descriptors import (
    ForwardManyToOneCacheDescriptor, ForwardOneToOneCacheDescriptor,
)
from cool.model.fields import ForeignKey, OneToOneField
from cool.model.models import AbstractUserMixin, BaseModel

__all__ = [
    'ForwardManyToOneCacheDescriptor', 'ForwardOneToOneCacheDescriptor',
    'ForeignKey', 'OneToOneField',
    'AbstractUserMixin', 'BaseModel',
]



# encoding: utf-8
import operator
from functools import reduce

from django.contrib.admin.utils import lookup_needs_distinct
from django.core.exceptions import FieldDoesNotExist
from django.db.models import Q
from django.db.models.constants import LOOKUP_SEP


def split_camel_name(name, fall=False):
    """
    驼峰命名分割为单词

    GenerateURLs => [Generate, URLs]
    generateURLsLite => [generate, URLs, Lite]
    """
    if not name:
        return []

    lastest_upper = name[0].isupper()
    idx_list = []
    for idx, char in enumerate(name):
        upper = char.isupper()
        # rising
        if upper and not lastest_upper:
            idx_list.append(idx)
        # falling
        elif fall and not upper and lastest_upper:
            idx_list.append(idx-1)
        lastest_upper = upper

    l_idx = 0
    name_items = []
    for r_idx in idx_list:
        if name[l_idx:r_idx]:
            name_items.append(name[l_idx:r_idx])
        l_idx = r_idx
    if name[l_idx:]:
        name_items.append(name[l_idx:])

    return name_items


def construct_search(queryset, field_name):
    """
    生成搜索关键字
    """
    if field_name.startswith('^'):
        return "%s__istartswith" % field_name[1:]
    elif field_name.startswith('='):
        return "%s__iexact" % field_name[1:]
    # Use field_name if it includes a lookup.
    opts = queryset.model._meta
    lookup_fields = field_name.split(LOOKUP_SEP)
    # Go through the fields, following all relations.
    prev_field = None
    for path_part in lookup_fields:
        if path_part == 'pk':
            path_part = opts.pk.name
        try:
            field = opts.get_field(path_part)
        except FieldDoesNotExist:
            # Use valid query lookups.
            if prev_field and prev_field.get_lookup(path_part):
                return field_name
        else:
            prev_field = field
            if hasattr(field, 'get_path_info'):
                # Update opts to follow the relation.
                opts = field.get_path_info()[-1].to_opts
    # Otherwise, use the field with icontains.
    return "%s__icontains" % field_name


def get_search_results(queryset, search_term, search_fields, model):
    """
    Return a tuple containing a queryset to implement the search
    and a boolean indicating if the results may contain duplicates.
    """
    use_distinct = False
    if search_fields and search_term:
        orm_lookups = [construct_search(queryset, str(search_field)) for search_field in search_fields]
        for bit in search_term.split():
            or_queries = [Q(**{orm_lookup: bit}) for orm_lookup in orm_lookups]
            queryset = queryset.filter(reduce(operator.or_, or_queries))
        use_distinct |= any(lookup_needs_distinct(model._meta, search_spec) for search_spec in orm_lookups)
    return queryset, use_distinct



# encoding: utf-8
import importlib
import inspect

from rest_framework import serializers
from rest_framework.fields import empty


class BaseSerializer(serializers.ModelSerializer):
    """
    序列化基类，文件字段会自动生成全路径url，property字段会用doc生成label或help_text
    """
    def __init__(self, instance=None, data=empty, **kwargs):
        if data is None:
            data = empty
        request = kwargs.pop("request", None)
        if request is not None:
            kwargs.setdefault("context", {}).update({"request": request})
        super().__init__(instance, data, **kwargs)
        root = self.root
        if root is not None:
            root._context = self._context
        for field in self._readable_fields:
            if isinstance(field, BaseSerializer) and not hasattr(field, '_context'):
                field._context = self._context

    def __new__(cls, *args, **kwargs):
        # We override this method in order to auto magically create
        # `ListSerializer` classes instead when `many=True` is set.
        request = kwargs.pop("request", None)
        if "context" not in kwargs and request is not None:
            kwargs["context"] = {"request": request}
        return super().__new__(cls, *args, **kwargs)

    def build_property_field(self, field_name, model_class):
        field_class, field_kwargs = super().build_property_field(field_name, model_class)
        label = getattr(model_class, field_name).__doc__
        if label:
            label = label.strip()
        if label:
            field_kwargs.setdefault('help_text' if '\n' in label else 'label', label)
        return field_class, field_kwargs

    @property
    def request(self):
        return self.context.get("request", None)


class RecursiveField(serializers.BaseSerializer):
    """
    树型结构序列化
    """

    # This list of attributes determined by the attributes that
    # `rest_framework.serializers` calls to on a field object
    PROXIED_ATTRS = (
        # methods
        'get_value',
        'get_initial',
        'run_validation',
        'get_attribute',
        'to_representation',

        # attributes
        'field_name',
        'source',
        'read_only',
        'default',
        'source_attrs',
        'write_only',
    )

    def __init__(self, to=None, **kwargs):
        """
        arguments:
        to - `None`, the name of another serializer defined in the same module
             as this serializer, or the fully qualified import path to another
             serializer. e.g. `ExampleSerializer` or
             `path.to.module.ExampleSerializer`
        """
        self.to = to
        self.init_kwargs = kwargs
        self._proxy = None

        # need to call super-constructor to support ModelSerializer
        super_kwargs = dict(
            (key, kwargs[key])
            for key in kwargs
            if key in inspect.signature(serializers.Field.__init__).parameters.keys()
        )
        super().__init__(**super_kwargs)

    def bind(self, field_name, parent):
        # Extra-lazy binding, because when we are nested in a ListField, the
        # RecursiveField will be bound before the ListField is bound
        setattr(self, 'bind_args', (field_name, parent))

    def find_parent_deep_number(self, field_class, check_subclass=False):
        obj = self
        deep_number = 0
        while obj is not None:
            if isinstance(obj, RecursiveField):
                obj = obj.proxy
            obj_class = type(obj)
            if obj_class == field_class or (check_subclass and issubclass(obj_class, field_class)):
                deep_number += 1
            obj = obj.parent
        return deep_number

    def get_parent_proxy(self, max_deep=2):
        ret = self
        if self.find_parent_deep_number(type(self.proxy)) <= max_deep:
            ret = ret.proxy
        return ret

    @property
    def proxy(self):
        if self._proxy is None:
            self._proxy = self.gen_proxy()
        return self._proxy

    def gen_proxy(self):
        if not self.bind_args:
            return None
        field_name, parent = self.bind_args

        if hasattr(parent, 'child') and parent.child is self:
            # RecursiveField nested inside of a ListField
            parent_class = parent.parent.__class__
        else:
            # RecursiveField directly inside a Serializer
            parent_class = parent.__class__

        assert issubclass(parent_class, serializers.BaseSerializer)

        if self.to is None:
            proxy_class = parent_class
        else:
            try:
                module_name, class_name = self.to.rsplit('.', 1)
            except ValueError:
                module_name, class_name = parent_class.__module__, self.to

            try:
                proxy_class = getattr(importlib.import_module(module_name), class_name)
            except Exception as e:
                raise ImportError('could not locate serializer %s' % self.to, e)

        # Create a new serializer instance and proxy it
        proxy = proxy_class(**self.init_kwargs)
        proxy.bind(field_name, parent)
        return proxy

    def __getattribute__(self, name):
        if name in RecursiveField.PROXIED_ATTRS:
            try:
                proxy = object.__getattribute__(self, 'proxy')
                return getattr(proxy, name)
            except AttributeError:
                pass

        return object.__getattribute__(self, name)



# encoding: utf-8

from rest_framework.response import Response

from cool.settings import cool_settings
from cool.views.error_code import ErrorCode


def get_response_dict(code, message, data, success_with_code_msg):
    if not success_with_code_msg and code == ErrorCode.SUCCESS:
        return data
    else:
        return {
            cool_settings.API_DEFAULT_CODE_KEY: code,
            cool_settings.API_DEFAULT_MESSAGE_KEY: message,
            cool_settings.API_DEFAULT_DATA_KEY: data,
        }


class ResponseData:
    """
    返回数据包装
    """
    def __init__(self, data, code=ErrorCode.SUCCESS, message=None, status_code=200,
                 success_with_code_msg=cool_settings.API_SUCCESS_WITH_CODE_MSG):
        self.code = code
        self.message = message
        self.data = data
        self.status_code = status_code
        self.success_with_code_msg = success_with_code_msg
        if isinstance(code, ErrorCode):
            self.code = code.code
            if self.message is None:
                self.message = code.desc

    def get_response_data(self):
        func = cool_settings.API_RESPONSE_DICT_FUNCTION
        return func(
            code=self.code, message=self.message, data=self.data, success_with_code_msg=self.success_with_code_msg
        )

    def get_response(self):
        return Response(data=self.get_response_data(), status=self.status_code)



# encoding: utf-8
import copy
from collections import OrderedDict
from importlib import import_module

from django.conf import settings
from django.core.validators import (
    BaseValidator, ProhibitNullCharactersValidator,
)
from django.db import models
from django.db.models import NOT_PROVIDED
from django.shortcuts import render
from django.template.loader import render_to_string
from django.utils.translation import gettext as _
from rest_framework import fields, serializers, validators
from rest_framework.fields import empty
from rest_framework.serializers import ModelSerializer
from rest_framework.utils import model_meta

from cool.views.error_code import ErrorCode
from cool.views.view import CoolBFFAPIView


def parse_validation_error(data):
    """
    参数检查结果处理
    """
    from django.core.exceptions import ValidationError
    from rest_framework.exceptions import (
        ValidationError as RestValidationError,
    )
    if isinstance(data, ValidationError):
        if hasattr(data, 'error_dict'):
            return parse_validation_error(dict(data))
        return parse_validation_error(list(data))
    elif isinstance(data, RestValidationError):
        return parse_validation_error(data.detail)
    elif isinstance(data, dict):
        return {key: parse_validation_error(value) for key, value in data.items()}
    elif isinstance(data, list):
        return [parse_validation_error(item) for item in data]
    else:
        return data


def get_rest_field_from_model_field(model, model_field, **kwargs):
    """
    通过model字段自动生成rest framework字段
    """
    if isinstance(model_field, models.Field):
        model_field = model_field.name
    s = ModelSerializer()
    info = model_meta.get_field_info(model)
    for field_name, relation_info in info.forward_relations.items():
        if relation_info.to_many:
            continue
        field = relation_info.model_field
        target_field = field
        verbose_name = [target_field.verbose_name]
        help_text = [target_field.help_text]
        while target_field.remote_field:
            target_field = target_field.remote_field.target_field
            verbose_name.append(target_field.verbose_name)
            help_text.append(target_field.help_text)

        target_field = copy.deepcopy(target_field)
        target_field.verbose_name = " - ".join(filter(lambda x: x, verbose_name))
        target_field.help_text = " - ".join(filter(lambda x: x, help_text))
        for attr in ('primary_key', 'blank', 'null', 'default', 'editable', 'serialize'):
            if hasattr(field, attr):
                setattr(target_field, attr, getattr(field, attr))
        info.fields_and_pk[field.name] = target_field
        try:
            info.fields_and_pk[field.attname] = target_field
        except AttributeError:
            pass
    field_info = info.fields_and_pk[model_field]
    field_class, field_kwargs = s.build_field(model_field, info, model, 0)
    field_kwargs.pop('read_only', None)
    gen_validators = field_kwargs.pop('validators', None)
    if gen_validators:
        gen_validators = list(filter(
            lambda x: not isinstance(x, (
                validators.UniqueValidator, validators.BaseUniqueForValidator, ProhibitNullCharactersValidator
            )),
            gen_validators
        ))
        if gen_validators:
            field_kwargs['validators'] = gen_validators
    field_kwargs = s.include_extra_kwargs(field_kwargs, kwargs)
    if not field_kwargs.get('required') and 'default' not in field_kwargs:
        field_kwargs['default'] = None if field_info.default is NOT_PROVIDED else field_info.default

    if field_class is fields.BooleanField and field_kwargs['default'] is None:
        field_class = fields.NullBooleanField
        field_kwargs.pop('allow_null', None)

    if not ('default' in kwargs or 'required' in kwargs or field_kwargs.get('required') or field_kwargs['default']):
        if (not field_kwargs.get('allow_null', False) and field_kwargs['default'] is None) \
                or (not field_kwargs.get('allow_blank', False) and not field_kwargs['default']):
            field_kwargs['required'] = True
            field_kwargs.pop('default')
    return field_class(**field_kwargs)


def get_field_info(field):
    """
    获取字段信息
    """
    field_type = field.__class__.__name__
    if field_type.endswith("Field") and field_type != 'Field':
        field_type = field_type[:-5]
    info = {
        '__field__': field,
        'label': str(field.label),
        'type': field_type,
        'default': field.default,
        'default_format': '' if field.default is empty else field.default,
        'required': field.required,
        'required_format': _('Yes') if field.required is True else _('No'),
        'help_text': field.help_text,
        'extend_info': OrderedDict()
    }
    field_validators = [field]
    field_validators.extend(getattr(field, 'validators', list()))
    validator_keys = ['max_value', 'min_value', 'max_length', 'min_length', 'max_digits', 'max_decimal_places',
                      'choices', 'regex', 'allowed_extensions', 'sep', 'child', 'is_list', 'children', 'serializer']
    for validator in field_validators:
        for k in validator_keys:
            v = getattr(validator, k, None)
            if v is not None:
                v = getattr(v, 'pattern', v)
                info['extend_info'].setdefault(k, list()).append(v)
        if isinstance(validator, BaseValidator):
            info['extend_info'].setdefault(validator.code, list()).append(validator.limit_value)
    for k in ['max_value', 'max_length', 'max_digits', 'max_decimal_places']:
        if k in info['extend_info']:
            info['extend_info'][k] = min(info['extend_info'][k])
    for k in ['min_value', 'min_length']:
        if k in info['extend_info']:
            info['extend_info'][k] = max(info['extend_info'][k])
    for k in info['extend_info']:
        if not isinstance(info['extend_info'][k], list):
            continue
        if len(info['extend_info'][k]) == 1:
            info['extend_info'][k] = info['extend_info'][k].pop()
    if 'choices' in info['extend_info']:
        info['extend_info'].pop('max_value', None)
        info['extend_info'].pop('min_value', None)

    def _format_info(_info):
        if isinstance(_info, list):
            return "[%s]" % _format_info(_info[0])
        if not isinstance(_info, dict):
            return ""
        if '__field__' in _info:
            return _("(%s %s default:%s,required:%s,%s %s)") % (
                _info['label'],
                _info['type'],
                _info['default_format'],
                _info['required_format'],
                _info['extend_info_format'],
                _info['help_text']
            )
        else:
            ret = "; ".join(["%s:%s" % (_k, _format_info(_v)) for _k, _v in _info.items()])
            return "{ %s }" % ret

    def _format_value(_value):
        if isinstance(_value, list):
            return ",".join(map(lambda x: _format_value(x) if isinstance(x, dict) else x, _value))
        elif isinstance(_value, dict):
            return ",".join(["%s:%s" % (_k, _format_value(_v)) for _k, _v in _value.items()])
        elif isinstance(_value, type) and issubclass(_value, serializers.Serializer):
            _info = get_serializer_field_info(_value())
            return _format_info(_info)
        elif isinstance(_value, fields.Field):
            _info = get_field_info(_value)
            return _format_info(_info)
        else:
            return _value

    info['extend_info_format'] = "; ".join(["%s:%s" % (k, _format_value(v)) for k, v in info['extend_info'].items()])
    return info


def get_serializer_field_info(serializer_obj, force_many=False):
    from . import serializer
    ret = dict()

    if isinstance(serializer_obj, serializer.RecursiveField):
        serializer_obj = serializer_obj.get_parent_proxy(2)

    if not isinstance(serializer_obj, serializer.RecursiveField):
        for field_name, field in serializer_obj.fields.items():
            if hasattr(field, 'fields'):
                ret[field_name] = get_serializer_field_info(field)
            elif hasattr(field, 'child'):
                ret[field_name] = get_serializer_field_info(field.child, force_many=True)
            elif hasattr(field, 'child_relation'):
                ret[field_name] = get_serializer_field_info(field.child_relation, force_many=True)
            else:
                ret[field_name] = get_field_info(field)

    return [ret] if force_many else ret


def get_list_info(serializer_obj):
    """
    获取列表序列化信息
    """
    from . import serializer
    child = serializer_obj.child

    if isinstance(child, serializer.RecursiveField):
        child = child.get_parent_proxy(2)

    if hasattr(child, 'fields'):
        return get_serializer_info(child, force_many=True)
    return [str(serializer_obj.label)]


def get_serializer_info(serializer_obj, force_many=False):
    """
    获取序列化信息
    """
    from . import serializer
    ret = dict()

    for field_name, field in serializer_obj.fields.items():
        if isinstance(field, serializer.RecursiveField):
            field = field.get_parent_proxy(2)

        if hasattr(field, 'fields'):
            ret[field_name] = get_serializer_info(field)
        elif hasattr(field, 'child'):
            ret[field_name] = get_list_info(field)
        elif hasattr(field, 'child_relation'):
            ret[field_name] = [str(field.child_relation.label)]
        else:
            ret[field_name] = str(field.label)
            if isinstance(field, fields.ChoiceField):
                choices = ",".join(["%s:%s" % (k, v) for k, v in field.choices.items()])
                if choices:
                    ret[field_name] += " (%s)" % choices
    return [ret] if force_many else ret


def get_url(head, urlpattern):
    """
    组合生成url
    """
    url = getattr(urlpattern, 'pattern', urlpattern).regex.pattern
    ret = head + url.replace('\\', '').rstrip("$?").lstrip('^')
    return ret.replace('//', '/')


def get_view_list(urlpattern=None, head='/', base_view=CoolBFFAPIView):
    """
    获取所有接口列表
    """
    ret = []
    if urlpattern is None:
        rooturl = import_module(settings.ROOT_URLCONF)
        for urlpattern in rooturl.urlpatterns:
            ret += get_view_list(urlpattern, get_url(head, urlpattern), base_view=base_view)
        return ret
    view_class = urlpattern
    for sub in ('callback', 'view_class'):
        view_class = getattr(view_class, sub, None)
        if view_class is None:
            break
    if view_class is not None and issubclass(view_class, base_view):
        retdict = dict()
        retdict['view_class'] = view_class
        retdict['params'] = view_class._meta.param_fields
        retdict['name'] = view_class().get_view_name()
        retdict['url'] = head.replace('//', '/').rstrip('/')
        ret.append(retdict)

    if hasattr(urlpattern, 'url_patterns'):
        for pattern in urlpattern.url_patterns:
            ret += get_view_list(pattern, get_url(head, pattern), base_view=base_view)

    return ret


def get_api_info(base_view=CoolBFFAPIView, base_params=(), add_base_view_params=True, exclude_views=()):
    """
    获取api接口信息

    :param base_view: 接口视图基类
    :param base_params: 公共参数
    :param add_base_view_params: 基类中参数增加到公共参数
    :param exclude_views:  排除接口视图
    """
    assert issubclass(base_view, CoolBFFAPIView)
    base_params = list(base_params)
    if add_base_view_params:
        opt = getattr(base_view, '_meta', None)
        param_fields = getattr(opt, 'param_fields', dict())
        for param_field in param_fields.keys():
            if param_field not in base_params:
                base_params.append(param_field)

    error_codes = ErrorCode.get_desc_dict()

    apis = list()
    for v in get_view_list(base_view=base_view):
        if issubclass(v['view_class'], exclude_views):
            continue
        has_file = False
        post = False
        length = 0
        no_len_count = 0
        for param, field in v['params'].items():
            if isinstance(field, fields.FileField):
                has_file = True
                post = True
            if param in ('pass', 'password'):
                post = True
            if isinstance(field, fields.CharField):
                if field.max_length is None:
                    no_len_count += 1
                else:
                    length += field.max_length

        if no_len_count > 3 or length > 200:
            post = True
        info = v['view_class'].get_view_info()
        base_params_num = 0
        for base_param in base_params:
            if base_param in info['request_info']:
                info['request_info'].move_to_end(base_param, False)
                base_params_num += 1
        params = list(info['request_info'].keys())[base_params_num:]
        apis.append({
            'name': v['name'],
            'url': v['url'],
            'ul_name': v['url'].replace('/', '_').strip('_'),
            'info': info,
            'self_params': params,
            'suggest_method': 'POST' if post else 'GET',
            'content_type': 'multipart/form-data' if has_file else 'application/x-www-form-urlencoded',
        })
    return {
        'base_params': base_params,
        'error_codes': error_codes,
        'apis': apis
    }


def get_api_doc(
    request=None,
    template_name='cool/views/api_doc.md',
    base_view=CoolBFFAPIView,
    exclude_params=(),
    exclude_base_view_params=True,
    exclude_views=()
):
    """
    生成api文档

    :param request: 请求request
    :param template_name: 接口模板
    :param base_view: 接口视图基类
    :param exclude_params: 排除参数
    :param exclude_base_view_params: 是否排除基类中参数
    :param exclude_views:  排除接口视图
    """
    api_info = get_api_info(base_view, exclude_params, exclude_base_view_params, exclude_views)
    api_info['server'] = request.build_absolute_uri("/")[:-1] if request is not None else '/'
    return render_to_string(template_name, api_info, request)


def get_api_doc_html(
    request,
    *args,
    md_template_name='cool/views/api_doc.md',
    base_view=CoolBFFAPIView,
    exclude_params=(),
    exclude_base_view_params=True,
    exclude_views=(),
    title=_('Api Document'),
    toc_left=True,
    **kwargs
):
    """
    生成api文档（markdown转html，依赖markdown）

    :param request: 请求request
    :param md_template_name: 接口模板
    :param base_view: 接口视图基类
    :param exclude_params: 排除参数
    :param exclude_base_view_params: 是否排除基类中参数
    :param exclude_views:  排除接口视图
    :param title: 文档标题
    :param toc_left: 宽浏览器中目录显示在左侧
    """
    md = get_api_doc(
        request=request,
        template_name=md_template_name,
        base_view=base_view,
        exclude_params=exclude_params,
        exclude_base_view_params=exclude_base_view_params,
        exclude_views=exclude_views
    )
    import markdown
    html = markdown.markdown(md, extensions=[
        'markdown.extensions.toc',
        'markdown.extensions.fenced_code',
        'markdown.extensions.tables'
    ])
    md_style_template_name = kwargs.get('md_style_template_name', 'cool/views/markdown.html')
    return render(request, md_style_template_name, context={'html': html, 'title': title, 'toc_left': toc_left})



# encoding: utf-8

from django.db import models

from cool.core import cache


class ModelCache(cache.BaseCache):
    """
    model缓存
    """
    key_prefix = 'cool:model_cache'
    default_timeout = 600

    pk = cache.CacheItem()
    unique = cache.CacheItem()

    def _get_key(self, model_cls, field_key, field_name):
        assert field_key is not None
        assert issubclass(model_cls, models.Model)
        if field_name is None:
            return "%s:%s" % (model_cls._meta.db_table, field_key), self.pk
        filed = model_cls._meta.get_field(field_name)
        assert filed.unique
        return "%s:%s:%s" % (model_cls._meta.db_table, filed.name, field_key), self.unique

    def get(self, model_cls, field_key, field_name=None, *, ttl=None):
        ret = self.get_many(model_cls, field_keys=[field_key], field_name=field_name, ttl=ttl)
        return ret.get(field_key, None)

    def get_many(self, model_cls, field_keys, field_name=None, *, ttl=None):
        field_keys = list(field_keys)
        keys_cache_to_model = dict()
        keys_model_to_cache = dict()
        item = None
        if ttl is None:
            ttl = self.default_timeout
        for field_key in field_keys:
            key, item = self._get_key(model_cls, field_key, field_name)
            keys_cache_to_model[key] = field_key
            keys_model_to_cache[field_key] = key
        ret = item.get_many(keys_cache_to_model.keys())
        if ret:
            ret = {keys_cache_to_model[k]: v for k, v in ret.items()}
        else:
            ret = dict()

        not_found = keys_model_to_cache.keys() - ret.keys()
        if field_name is None:
            field_name = 'pk'
        if not_found:
            not_found_info = dict()
            if hasattr(model_cls, 'get_queryset') and callable(model_cls.get_queryset):
                queryset = model_cls.get_queryset()
            else:
                queryset = model_cls.objects
            queryset = queryset.filter(**{"%s__in" % field_name: not_found})
            for obj in queryset:
                not_found_info[getattr(obj, field_name)] = obj
            item.set_many({keys_model_to_cache[k]: v for k, v in not_found_info.items()}, ttl)
            ret.update(not_found_info)
        # 按照参数传递顺序返回数据
        return {field_key: ret[field_key] for field_key in field_keys if field_key in ret}

    def delete(self, model_cls, field_key, field_name=None):
        key, item = self._get_key(model_cls, field_key, field_name)
        return item.delete(key)

    def delete_many(self, model_cls, field_keys, field_name=None):
        keys = list()
        item = None
        for field_key in field_keys:
            key, item = self._get_key(model_cls, field_key, field_name)
            keys.append(key)
        if item is None:
            return None
        return item.delete_many(keys)


model_cache = ModelCache()



# encoding: utf-8

from django.core.exceptions import ValidationError
from django.db import transaction
from django.utils.translation import gettext as _, gettext_lazy
from rest_framework import fields
from rest_framework.exceptions import ValidationError as RestValidationError
from rest_framework.utils import model_meta

from cool.core.utils import get_search_results
from cool.views import CoolAPIException, ErrorCode
from cool.views.fields import JSONCheckField, SplitCharField
from cool.views.utils import (
    get_rest_field_from_model_field, parse_validation_error,
)


class PageMixin:
    """
    分页返回数据Mixin
    """
    PAGE_SIZE_MAX = 200
    DEFAULT_PAGE_SIZE = 100

    @classmethod
    def get_extend_param_fields(cls):
        assert 0 < cls.DEFAULT_PAGE_SIZE <= cls.PAGE_SIZE_MAX, (
            "DEFAULT_PAGE_SIZE mast between 0 and PAGE_SIZE_MAX in class %s" % cls.__name__
        )
        return super().get_extend_param_fields() + (
            (
                'page', fields.IntegerField(
                    label=gettext_lazy('Page number'),
                    default=1,
                    help_text=gettext_lazy('Start with %(start)s') % {'start': 1}
                )
            ),
            (
                'page_size', fields.IntegerField(
                    label=gettext_lazy('Page size'),
                    default=cls.DEFAULT_PAGE_SIZE,
                    min_value=1,
                    max_value=cls.PAGE_SIZE_MAX
                )
            ),
        )

    @classmethod
    def response_info_data(cls):
        return {
            'page_size': _('Page size'),
            'list': [super().response_info_data()],
            'page': _('Page number'),
            'total_page': _('Total page'),
            'total_data': _('Total data')
        }

    def get_page_context(self, request, queryset, serializer_cls):
        page_size = request.params.page_size
        total_data = queryset.count()
        total_page = (total_data + page_size - 1) // page_size
        page = request.params.page
        data = []
        if total_data > 0 and 1 <= page <= total_page:
            start = (page - 1) * page_size
            data = serializer_cls(queryset[start:start + page_size], request=request, many=True).data

        return {'page_size': page_size, 'list': data, 'page': page, 'total_page': total_page, 'total_data': total_data}


class CRIDMixin:
    """
    class ObjectAddDMixin(Add, APIBase):
        model = models.Object
        add_fields = ['name', 'desc']
    """
    model = None

    @classmethod
    def get_model_field_info(cls):
        if not hasattr(cls, '_model_field_info'):
            setattr(cls, '_model_field_info', model_meta.get_field_info(cls.model))
        return getattr(cls, '_model_field_info')


class SearchListMixin(PageMixin, CRIDMixin):
    PAGE_SIZE_MAX = 1000
    model = None
    order_field = ('-pk', )

    @classmethod
    def get_extend_param_fields(cls):
        """
        添加搜索字段
        """
        ret = list()
        ret.extend(super().get_extend_param_fields())
        ret.append(('search_term', fields.CharField(label=_('Search key'), default='')))
        return tuple(ret)

    @property
    def name(self):
        """
        API文档中view的名字
        """
        return _("{model_name} List").format(model_name=self.model._meta.verbose_name)

    def get_search_fields(self):
        """
        返回本model可以被搜索的字段集合（基类回自动将带索引的字段生成搜索字段集合）
        """
        return self.model.get_search_fields()

    def get_queryset(self, request, queryset=None):
        if queryset is None:
            queryset = self.model.objects.order_by(*self.order_field)
        if request.params.search_term:
            # 筛选搜索关键词
            queryset, use_distinct = get_search_results(
                queryset, request.params.search_term, self.get_search_fields(), self.model
            )
            if use_distinct:
                queryset = queryset.distinct()
        return queryset

    def get_context(self, request, *args, **kwargs):
        return self.get_page_context(request, self.get_queryset(request), self.response_info_serializer_class)


class InfoMixin(CRIDMixin):
    model = None
    pk_id = True
    ex_unique_ids = []

    @property
    def name(self):
        return _("{model_name} Info").format(model_name=self.model._meta.verbose_name)

    @classmethod
    def get_extend_param_fields(cls):
        ret = list()
        ret.extend(super().get_extend_param_fields())
        if cls.model is not None:
            num = len(cls.ex_unique_ids)
            info = cls.get_model_field_info()
            if cls.pk_id:
                num += 1
                ret.append((info.pk.name, get_rest_field_from_model_field(
                    cls.model, info.pk.name, **{'default': None} if num > 1 else {'required': True}
                )))
            for ex_unique_id in cls.ex_unique_ids:
                assert ex_unique_id in info.fields_and_pk and info.fields_and_pk[ex_unique_id].unique, (
                    "Field %s not found in %s's unique fields" % (ex_unique_id, cls.model.__name__)
                )
                ret.append((ex_unique_id, get_rest_field_from_model_field(
                    cls.model, ex_unique_id, **{'default': None} if num > 1 else {'required': True}
                )))
            assert num > 0, 'Must set unique fields to ex_unique_ids or set True to pk_id'

        return tuple(ret)

    def get_obj(self, request, queryset=None):
        if queryset is None:
            queryset = self.model.objects.all()
        blank = True
        param_fields = [self.get_model_field_info().pk.name]
        param_fields.extend(self.ex_unique_ids)
        for field_name in param_fields:
            field = getattr(request.params, field_name)
            if field is not None:
                blank = False
                queryset = queryset.filter(**{field_name: field})
        if blank:
            raise CoolAPIException(
                ErrorCode.ERROR_BAD_PARAMETER,
                data=_("{fields} cannot be empty at the same time").format(fields=",".join(param_fields))
            )
        return queryset.first()

    def get_context(self, request, *args, **kwargs):
        return self.response_info_serializer_class(self.get_obj(request), request=request).data


class AddMixin(CRIDMixin):
    add_fields = []

    @property
    def name(self):
        return _("Add {model_name}").format(model_name=self.model._meta.verbose_name)

    @classmethod
    def get_extend_param_fields(cls):
        ret = list()
        ret.extend(super().get_extend_param_fields())
        if cls.model is not None:
            for add_field in cls.add_fields:
                field = get_rest_field_from_model_field(cls.model, add_field)
                ret.append((add_field, field))
        return tuple(ret)

    def init_fields(self, request, obj):
        for add_field in self.add_fields:
            value = getattr(request.params, add_field, None)
            if value is not None:
                setattr(obj, add_field, value)

    def save_obj(self, request, obj):
        obj.full_clean()
        obj.save(force_insert=True)

    def serializer_response(self, data, request):
        return self.response_info_serializer_class(data, request=request).data

    def get_context(self, request, *args, **kwargs):
        with transaction.atomic():
            try:
                obj = self.model()
                self.init_fields(request, obj)
                self.save_obj(request, obj)
            except ValidationError as e:
                raise RestValidationError(parse_validation_error(e))
        return self.serializer_response(obj, request=request)


class EditMixin(CRIDMixin):
    unique_key = 'pk'
    edit_fields = []

    @property
    def name(self):
        return _("Edit {model_name}").format(model_name=self.model._meta.verbose_name)

    @classmethod
    def get_extend_param_fields(cls):
        ret = list()
        ret.extend(super().get_extend_param_fields())
        field = cls.get_model_field_info().fields_and_pk[cls.unique_key]
        assert field.unique, "Field %s is not unique" % cls.unique_key
        ret.append((field.name, get_rest_field_from_model_field(cls.model, field, required=True)))
        if cls.model is not None:
            for edit_field in cls.edit_fields:
                ret.append((edit_field, get_rest_field_from_model_field(cls.model, edit_field, default=None)))
        return tuple(ret)

    def get_obj(self, request):
        return self.model.get_obj_by_pk_from_cache(request.params.id)

    def modify_obj(self, request, obj):
        for edit_field in self.edit_fields:
            value = getattr(request.params, edit_field, None)
            if value is not None:
                setattr(obj, edit_field, value)

    def save_obj(self, request, obj):
        obj.full_clean()
        obj.save_changed()

    def serializer_response(self, data, request):
        return self.response_info_serializer_class(data, request=request).data

    def get_context(self, request, *args, **kwargs):
        with transaction.atomic():
            obj = self.get_obj(request)
            self.modify_obj(request, obj)
            self.save_obj(request, obj)
        return self.serializer_response(obj, request=request)


class DeleteMixin(CRIDMixin):
    unique_key = 'pk'
    unique_key_sep = ','

    @property
    def name(self):
        return _("Delete {model_name}").format(model_name=self.model._meta.verbose_name)

    @classmethod
    def get_extend_param_fields(cls):
        ret = list()
        ret.extend(super().get_extend_param_fields())
        field = cls.get_model_field_info().fields_and_pk[cls.unique_key]
        assert field.unique, "Field %s is not unique" % cls.unique_key
        ret.append((field.name + 's', SplitCharField(
            label=_('Primary keys'),
            sep=cls.unique_key_sep,
            child=get_rest_field_from_model_field(cls.model, field, required=True)
        )))
        return tuple(ret)

    def get_queryset(self, request):
        return self.model.objects.filter(id__in=request.params.ids)

    def delete_object(self, request, obj):
        obj.delete()

    def delete_queryset(self, request, queryset):
        for obj in queryset:
            self.delete_object(request, obj)

    def get_context(self, request, *args, **kwargs):
        with transaction.atomic():
            queryset = self.get_queryset(request)
            self.delete_queryset(request, queryset)
        return None


class ExtJSONCheckField(JSONCheckField):

    def __init__(self, *args, **kwargs):
        self.ext_model_field_key = kwargs.pop('ext_model_field_key')
        assert isinstance(self.ext_model_field_key, ExtModelFieldKey)
        super().__init__(*args, **kwargs)

    def clean_dict_data(self, data):
        data = super().clean_dict_data(data)
        if self.ext_model_field_key.pk_field is None:
            return data
        errors = dict()
        if data.get(self.ext_model_field_key.pk_field, None) is not None:
            for field_name in self.ext_model_field_key.add_field_list:
                if field_name in self.ext_model_field_key.add_not_required_field_list:
                    continue
                if data.get(field_name, None) is None:
                    try:
                        self.children[field_name].fail('required')
                    except RestValidationError as e:
                        errors[field_name] = e.detail
        if errors:
            raise RestValidationError(errors)
        return data

    def run_children_validation(self, data):
        return super().run_children_validation(data)


class ExtModelFieldKey:
    def __init__(
            self,
            field_name,
            ext_model,
            ext_foreign_key,
            edit_field_list=(),
            add_field_list=(),
            add_not_required_field_list=(),
            add_default_fields=None,
            pk_field='id',
            delete_not_found=True,
    ):
        """
        生成 ext_model_fields 供 get_ext_model_fields 使用

        :param field_name: 接口字段名
        :param ext_model: 扩展model类型
        :param ext_foreign_key: 扩展model中的外键字段
        :param add_field_list: 添加字段列表
        :param edit_field_list: 修改字段列表
        :param add_not_required_field_list: 新增非必填参数
        :param add_default_fields: 添加时默认值
        :param pk_field: 修改key（必须为主键或唯一键，提交数据有该值为修改，没有为新增）,设空不允许修改
        :param delete_not_found: 是否删除未出现在列表中的数据
        """
        if add_default_fields is None:
            add_default_fields = dict()
        self.field_name = field_name
        self.ext_model = ext_model
        self.ext_foreign_key = ext_foreign_key
        self.add_field_list = add_field_list
        self.edit_field_list = edit_field_list
        self.add_not_required_field_list = add_not_required_field_list
        self.add_default_fields = add_default_fields
        self.pk_field = pk_field
        self.delete_not_found = delete_not_found

    def get_json_check_field(self, label):
        children = dict()
        add_only = self.pk_field is None
        if not add_only:
            children[self.pk_field] = get_rest_field_from_model_field(
                self.ext_model, self.pk_field, default=None,
                help_text=_('If the parameter has a value, it is modified; if it has no value, it is added')
            )
        field_list = list()
        field_list.extend(self.add_field_list)
        if not add_only:
            field_list.extend(self.edit_field_list)
        for field in field_list:
            if field in children:
                continue
            add_not_required = field in self.add_not_required_field_list
            field_kwargs = dict()
            if not add_only or field in self.add_not_required_field_list:
                field_kwargs['default'] = None
            if not add_only:
                help_text = []
                if field in self.add_field_list:
                    help_text.append(_('not required when add') if add_not_required else _('required when add'))
                if field in self.edit_field_list:
                    help_text.append(_('not required when edit'))
                field_kwargs['help_text'] = ",".join(help_text)

            children[field] = get_rest_field_from_model_field(
                self.ext_model, field, **field_kwargs
            )
        return ExtJSONCheckField(
            label=label,
            children=children,
            is_list=True,
            default=None,
            ext_model_field_key=self
        )

    def gen_objs(self, data, obj, get_ext_obj):
        params = dict()
        params[self.ext_foreign_key] = obj
        add_objs = []
        edit_objs = []
        edit_ids = []
        param_errors = dict()
        for idx, p in enumerate(data):
            if self.pk_field is not None and self.pk_field in p and p[self.pk_field] is not None:
                if p[self.pk_field] in edit_ids:
                    param_errors[idx] = ValidationError(_('Primary key duplicate'))
                    continue
                edit_ids.append(p[self.pk_field])
                obj = get_ext_obj(self.ext_model, self.pk_field, p[self.pk_field])
                if obj is None:
                    param_errors[idx] = ValidationError(_('Modification item not found'))
                    continue
                for ext_key, ext_value in params.items():
                    if getattr(obj, ext_key) == ext_value:
                        param_errors[idx] = ValidationError(_('Modification item not found'))
                        break
                else:
                    for key, value in p.items():
                        if key in self.edit_field_list and value is not None:
                            setattr(obj, key, value)
                    try:
                        obj.full_clean()
                    except ValidationError as e:
                        param_errors[idx] = e
                    edit_objs.append(obj)
            else:
                obj = self.ext_model(
                    **self.add_default_fields,
                    **params,
                    **dict(filter(lambda x: x[0] in self.add_field_list, p.items())),
                )
                try:
                    obj.full_clean()
                except ValidationError as e:
                    param_errors[idx] = e
                add_objs.append(obj)
        del_objs = []
        if self.delete_not_found:
            queryset = self.ext_model.objects.filter(**params)
            if edit_ids:
                queryset = queryset.exclude({"%s__in" % self.pk_field: edit_ids})
            del_objs = list(queryset)
        if param_errors:
            raise RestValidationError(parse_validation_error(param_errors))
        return add_objs, edit_objs, del_objs


class ExtManyToOneMixin:

    @classmethod
    def get_ext_model_fields(cls):
        """
        [
            ExtModelFieldKey()
        ]
        """
        return ()

    def get_ext_obj(self, ext_model, unique_field, unique_field_value):
        return ext_model.get_obj_by_unique_key_from_cache(**{unique_field: unique_field_value})

    def delete_ext_obj(self, obj):
        obj.delete()

    def delete_ext_objs(self, objs):
        for obj in objs:
            self.delete_ext_obj(obj)

    def save_ext_obj(self, obj):
        obj.save_changed()

    def edit_ext_objs(self, objs):
        for obj in objs:
            self.save_ext_obj(obj)

    def add_ext_objs(self, ext_model, objs):
        ext_model.objects.bulk_create(objs)

    def save_obj(self, request, obj):
        super().save_obj(request, obj)
        self.do_ext(request, obj)

    def do_ext(self, request, obj):
        errors = dict()
        ex_objs = list()
        for model_fields in self.get_ext_model_fields():
            param = getattr(request.params, model_fields.field_name)
            if param is None:
                continue

            def _get_ext_obj(*args, **kwargs):
                self.get_ext_obj(*args, **kwargs)

            try:
                add_objs, edit_objs, del_objs = model_fields.gen_objs(param, obj, _get_ext_obj)
                ex_objs.append((model_fields.ext_model, add_objs, edit_objs, del_objs))
            except RestValidationError as e:
                errors[model_fields.field_name] = e
        if errors:
            raise RestValidationError(parse_validation_error(errors))
        for ext_model, add_objs, edit_objs, del_objs in ex_objs:
            if del_objs:
                self.delete_ext_objs(del_objs)
            if edit_objs:
                self.edit_ext_objs(edit_objs)
            if add_objs:
                self.add_ext_objs(ext_model, add_objs)

    @classmethod
    def get_extend_param_fields(cls):
        ret = list()
        ret.extend(super().get_extend_param_fields())
        for model_fields in cls.get_ext_model_fields():
            ret.append((model_fields.field_name, model_fields.get_json_check_field(
                label="{model_name} List".format(model_name=model_fields.ext_model._meta.verbose_name)
            )))
        return tuple(ret)
